@@ -16,7 +16,8 @@ from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
+
+import logging
 
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -35,6 +36,8 @@ from gajim.gtk.menus import escape_mnemonic
 EMOJI_NUM_GENDERS = len(['f', 'm', 'n'])
 MENUS_MAX_ENTRIES = 2 * EMOJI_NUM_GENDERS
 
+log = logging.getLogger('gajim.gtk.chat_action_processor')
+
 
 class ChatActionProcessor(Gtk.Popover):
     def __init__(self, message_input: Gtk.TextView) -> None:
@@ -48,8 +51,8 @@ class ChatActionProcessor(Gtk.Popover):
         self.connect('closed', self._on_popover_closed)
         self.connect('destroy', self._on_destroy)
 
-        self._account: Optional[str] = None
-        self._contact: Optional[types.ChatContactT] = None
+        self._account: str | None = None
+        self._contact: types.ChatContactT | None = None
 
         self._message_input = message_input
         self._message_input.connect('key-press-event', self._on_key_press)
@@ -60,8 +63,8 @@ class ChatActionProcessor(Gtk.Popover):
 
         self._nick_completion = GroupChatNickCompletion()
 
-        self._start_mark: Optional[Gtk.TextMark] = None
-        self._current_iter: Optional[Gtk.TextIter] = None
+        self._start_mark: Gtk.TextMark | None = None
+        self._current_iter: Gtk.TextIter | None = None
 
         self._active = False
 
@@ -107,9 +110,18 @@ class ChatActionProcessor(Gtk.Popover):
         return False
 
     def _on_focus_out(self, *args: Any) -> bool:
-        # Add 100 ms timeout in order to process potential menu item click
-        # events (which will emit focus-out-event on the message input)
-        GLib.timeout_add(100, self.popdown)
+        win = self.get_window()
+        default_display = Gdk.Display.get_default()
+        if default_display is not None:
+            pointer = default_display.get_default_seat().get_pointer()
+            if win is not None and pointer is not None:
+                _win, x_pos, y_pos, _mod = win.get_device_position(pointer)
+                rect = self.get_allocation()
+                if (x_pos not in range(rect.width) or
+                        y_pos not in range(rect.height)):
+                    # Only popdown if click is outside of Popover's Rectangle
+                    self.popdown()
+
         return False
 
     def _on_popover_closed(self, _popover: Gtk.Popover) -> None:
@@ -240,21 +252,34 @@ class ChatActionProcessor(Gtk.Popover):
         self._menu.remove_all()
         emoji_data = get_emoji_data()
 
+        sn_matches: dict[str, str] = {}
+        kw_matches: dict[str, str] = {}
+
         for keyword, entries in emoji_data.items():
-            for short_name in entries:
-                if not keyword.startswith(action_text):
-                    continue
-                emoji = entries[short_name]
+            if not keyword.startswith(action_text):
+                continue
+            for short_name, emoji in entries.items():
                 label = f'{emoji} {short_name}'
-                if keyword != short_name:
-                    label += f'  [{keyword}]'
-                action_data = GLib.Variant('s', emoji)
-                menu_item = Gio.MenuItem()
-                menu_item.set_label(escape_mnemonic(label))
-                menu_item.set_attribute_value('action-data', action_data)
-                self._menu.append_item(menu_item)
-                if self._menu.get_n_items() >= MENUS_MAX_ENTRIES:
-                    break
+                if keyword == short_name:
+                    # Replace a possible keyword match with the shortname match:
+                    sn_matches[emoji] = label
+                    if kw_matches.get(emoji) is not None:
+                        del kw_matches[emoji]
+                else:
+                    # Only add a keyword match if no shortname match:
+                    if sn_matches.get(emoji) is None:
+                        kw_matches[emoji] = f'{label}  [{keyword}]'
+
+        log.debug('Found %d "%s…" emoji by short name, %d more by keyword',
+                  len(sn_matches), action_text, len(kw_matches))
+
+        # Put all shortname matches before keyword matches:
+        for emoji, label in list(sn_matches.items()) + list(kw_matches.items()):
+            action_data = GLib.Variant('s', emoji)
+            menu_item = Gio.MenuItem()
+            menu_item.set_label(escape_mnemonic(label))
+            menu_item.set_attribute_value('action-data', action_data)
+            self._menu.append_item(menu_item)
             if self._menu.get_n_items() >= MENUS_MAX_ENTRIES:
                 break
 
