@@ -17,12 +17,10 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
-from typing import Iterator
 from typing import NamedTuple
-from typing import Optional
-from typing import Union
 
+from collections.abc import Callable
+from collections.abc import Iterator
 from enum import Enum
 from enum import IntEnum
 from enum import unique
@@ -33,7 +31,7 @@ from gajim.common.setting_values import AllSettingsT
 
 class Filter(NamedTuple):
     name: str
-    pattern: Union[str, list[str]]
+    pattern: str | list[str]
     default: bool
 
 
@@ -41,15 +39,15 @@ class Setting(NamedTuple):
     kind: SettingKind
     label: str
     type: SettingType
-    value: Optional[AllSettingsT] = None
-    name: Optional[str] = None
-    callback: Optional[Callable[..., None]] = None
-    data: Optional[Any] = None
-    desc: Optional[str] = None
-    bind: Optional[str] = None
-    inverted: Optional[bool] = None
-    enabled_func: Optional[Callable[..., bool]] = None
-    props: Optional[dict[str, Any]] = None
+    value: AllSettingsT | None = None
+    name: str | None = None
+    callback: Callable[..., None] | None = None
+    data: Any | None = None
+    desc: str | None = None
+    bind: str | None = None
+    inverted: bool | None = None
+    enabled_func: Callable[..., bool] | None = None
+    props: dict[str, Any] | None = None
 
 
 DEFAULT_WORKSPACE_COLOR = 'rgb(191,15,167)'
@@ -192,6 +190,7 @@ APP_ACTIONS = [
     ('copy-text', 's'),
     ('create-groupchat', 's'),
     ('faq', None),
+    ('privacy-policy', None),
     ('features', None),
     ('file-transfer', None),
     ('forget-groupchat', 'a{sv}'),
@@ -231,7 +230,7 @@ MAIN_WIN_ACTIONS = [
     ('add-to-roster', None, True),
     ('start-voice-call', None, False),
     ('start-video-call', None, False),
-    ('show-contact-info', None, False),
+    ('show-contact-info', None, True),
     ('send-message', None, False),
     ('muc-change-nickname', None, False),
     ('muc-invite', None, False),
@@ -271,6 +270,9 @@ MAIN_WIN_ACTIONS = [
     ('switch-workspace-7', None, True),
     ('switch-workspace-8', None, True),
     ('switch-workspace-9', None, True),
+    ('increase-app-font-size', None, True),
+    ('decrease-app-font-size', None, True),
+    ('reset-app-font-size', None, True),
     ('toggle-chat-list', None, True),
     ('add-workspace', 's', True),
     ('edit-workspace', 's', True),
